@@ -126,13 +126,11 @@ typedef enum nsd_rc nsd_rc_type;
 #define TYPE_NSEC	47	/* RFC 4033, 4034, and 4035 */
 #define TYPE_DNSKEY	48	/* RFC 4033, 4034, and 4035 */
 #define TYPE_DHCID	49	/* RFC4701 DHCP information */
+#define TYPE_NSEC3	50	/* NSEC3, secure denial, prevents zonewalking */
+#define TYPE_NSEC3PARAM 51	/* NSEC3PARAM at zone apex nsec3 parameters */
 
 #define TYPE_SPF        99      /* RFC 4408 */
 
-/* high type range RRTYPES */
-#define TYPE_NSEC3	65324	/* NSEC3, dns secure denial, prevents zonewalking */
-#define TYPE_NSEC3PARAM 65325	/* NSEC3PARAM at zone apex nsec3 parameters */
-
 #define TYPE_TSIG	250
 #define TYPE_IXFR	251
 #define TYPE_AXFR	252
@@ -162,7 +160,6 @@ enum rdata_wireformat
 	RDATA_WF_UNCOMPRESSED_DNAME, /* Uncompressed domain name.  */
 	RDATA_WF_BYTE,		     /* 8-bit integer.  */
 	RDATA_WF_SHORT,		     /* 16-bit integer.  */
-	RDATA_WF_24BIT,              /* 24-bit integer. */
 	RDATA_WF_LONG,		     /* 32-bit integer.  */
 	RDATA_WF_TEXT,		     /* Text string.  */
 	RDATA_WF_A,		     /* 32-bit IPv4 address.  */
@@ -183,7 +180,6 @@ enum rdata_zoneformat
 	RDATA_ZF_TEXT,		/* Text string.  */
 	RDATA_ZF_BYTE,		/* 8-bit integer.  */
 	RDATA_ZF_SHORT,		/* 16-bit integer.  */
-	RDATA_ZF_24BIT,		/* 24-bit integer.  */
 	RDATA_ZF_LONG,		/* 32-bit integer.  */
 	RDATA_ZF_A,		/* 32-bit IPv4 address.  */
 	RDATA_ZF_AAAA,		/* 128-bit IPv6 address.  */
@@ -223,11 +219,9 @@ typedef struct rrtype_descriptor rrtype_descriptor_type;
  * Indexed by type.  The special type "0" can be used to get a
  * descriptor for unknown types (with one binary rdata).
  *
- * spf + 1 + the number of high-type-range RRTYPES
+ * spf + 1
  */
-#define RRTYPE_DESCRIPTORS_LENGTH  (TYPE_SPF + 1 + 2)
-/* below this value the types are consecutive numbered */
-#define RRTYPE_DESCRIPTORS_IDX_LEN (TYPE_SPF + 1)
+#define RRTYPE_DESCRIPTORS_LENGTH  (TYPE_SPF + 1)
 rrtype_descriptor_type *rrtype_descriptor_by_name(const char *name);
 rrtype_descriptor_type *rrtype_descriptor_by_type(uint16_t type);
 
