@@ -1,5 +1,5 @@
 /*
- * edns.h -- EDNS definitions (RFC 2671).
+ * edns.c -- EDNS definitions (RFC 2671).
  *
  * Copyright (c) 2001-2006, NLnet Labs. All rights reserved.
  *
@@ -25,7 +25,7 @@ edns_init_data(edns_data_type *data, uint16_t max_length)
 	/* udp payload size */
 	data->ok[3] = (max_length & 0xff00) >> 8; /* size_hi */
 	data->ok[4] = max_length & 0x00ff;	  /* size_lo */
-	
+
 	data->error[1] = (TYPE_OPT & 0xff00) >> 8;	/* type_hi */
 	data->error[2] = TYPE_OPT & 0x00ff;		/* type_lo */
 	data->error[3] = (max_length & 0xff00) >> 8;	/* size_hi */
@@ -36,15 +36,15 @@ edns_init_data(edns_data_type *data, uint16_t max_length)
 void
 edns_init_nsid(edns_data_type *data, uint16_t nsid_len)
 {
-	/* add nsid length bytes */
-	data->rdata_nsid[0] = ((OPT_HDR + nsid_len) & 0xff00) >> 8; /* length_hi */
-	data->rdata_nsid[1] = ((OPT_HDR + nsid_len) & 0x00ff);      /* length_lo */
+       /* add nsid length bytes */
+       data->rdata_nsid[0] = ((OPT_HDR + nsid_len) & 0xff00) >> 8; /* length_hi */
+       data->rdata_nsid[1] = ((OPT_HDR + nsid_len) & 0x00ff);      /* length_lo */
 
-	/* NSID OPT HDR */
-	data->nsid[0] = (NSID_CODE & 0xff00) >> 8;
-	data->nsid[1] = (NSID_CODE & 0x00ff);
-	data->nsid[2] = (nsid_len & 0xff00) >> 8;
-	data->nsid[3] = (nsid_len & 0x00ff);
+       /* NSID OPT HDR */
+       data->nsid[0] = (NSID_CODE & 0xff00) >> 8;
+       data->nsid[1] = (NSID_CODE & 0x00ff);
+       data->nsid[2] = (nsid_len & 0xff00) >> 8;
+       data->nsid[3] = (nsid_len & 0x00ff);
 }
 
 void
@@ -88,11 +88,12 @@ edns_parse_record(edns_record_type *edns, buffer_type *packet)
 	opt_version = buffer_read_u8(packet);
 	opt_flags = buffer_read_u16(packet);
 	opt_rdlen = buffer_read_u16(packet);
-
-	if (opt_rdlen != 0 || opt_version != 0) {
+	
+	if (opt_version != 0) {
 		edns->status = EDNS_ERROR;
 		return 1;
 	}
+
 	if (opt_rdlen > 0) {
 		/* there is more to come, read opt code
 		 * should be NSID - there are no others */
@@ -103,7 +104,7 @@ edns_parse_record(edns_record_type *edns, buffer_type *packet)
 
 	edns->status = EDNS_OK;
 	edns->maxlen = opt_class;
-	edns->dnssec_ok = !!(opt_flags & DNSSEC_OK_MASK);
+	edns->dnssec_ok = opt_flags & DNSSEC_OK_MASK;
 	return 1;
 }
 
