@@ -40,11 +40,12 @@ allocate_domain_info(domain_table_type *table,
 	result->rrsets = NULL;
 	result->number = 0;
 #ifdef NSEC3
-	result->nsec3_exact = NULL;
 	result->nsec3_cover = NULL;
 	result->nsec3_wcard_child_cover = NULL;
-	result->nsec3_ds_parent_exact = NULL;
 	result->nsec3_ds_parent_cover = NULL;
+	result->nsec3_lookup = NULL;
+	result->nsec3_is_exact = 0;
+	result->nsec3_ds_parent_is_exact = 0;
 #endif
 	result->is_existing = 0;
 	result->is_apex = 0;
@@ -72,11 +73,12 @@ domain_table_create(region_type *region)
 	root->is_existing = 0;
 	root->is_apex = 0;
 #ifdef NSEC3
-	root->nsec3_exact = NULL;
+	root->nsec3_is_exact = 0;
+	root->nsec3_ds_parent_is_exact = 0;
 	root->nsec3_cover = NULL;
 	root->nsec3_wcard_child_cover = NULL;
-	root->nsec3_ds_parent_exact = NULL;
 	root->nsec3_ds_parent_cover = NULL;
+	root->nsec3_lookup = NULL;
 #endif
 	
 	result = (domain_table_type *) region_alloc(region,
@@ -188,21 +190,25 @@ domain_table_insert(domain_table_type *table,
 	return result;
 }
 
-void
+int
 domain_table_iterate(domain_table_type *table,
 		    domain_table_iterator_type iterator,
 		    void *user_data)
 {
 	const void *dname;
 	void *node;
+	int error = 0;
 
 	assert(table);
 
 	RBTREE_WALK(table->names_to_domains, dname, node) {
-		iterator((domain_type *) node, user_data);
+		error += iterator((domain_type *) node, user_data);
 	}
+
+	return error;
 }
 
+
 void
 domain_add_rrset(domain_type *domain, rrset_type *rrset)
 {
