@@ -39,6 +39,8 @@ struct nsd_options {
 	ip_address_option_t* ip_addresses;
 
 	int debug_mode;
+	int verbosity;
+	int hide_version;
 	int ip4_only;
 	int ip6_only;
 	const char* database;
