@@ -15,12 +15,16 @@
 #define	MAXTOKENSLEN	512		/* Maximum number of tokens per entry */
 #define	B64BUFSIZE	65535		/* Buffer size for b64 conversion */
 #define	ROOT		(const uint8_t *)"\001"
-#define	MAXINCLUDES	10
 
 #define NSEC_WINDOW_COUNT     256
 #define NSEC_WINDOW_BITS_COUNT 256
 #define NSEC_WINDOW_BITS_SIZE  (NSEC_WINDOW_BITS_COUNT / 8)
 
+#define IPSECKEY_NOGATEWAY      0       /* RFC 4025 */
+#define IPSECKEY_IP4            1
+#define IPSECKEY_IP6            2
+#define IPSECKEY_DNAME          3
+
 #define LINEBUFSZ 1024
 
 struct lex_data {
@@ -29,7 +33,6 @@ struct lex_data {
 };
 
 #define DEFAULT_TTL 3600
-#define MAXINCLUDES 10
 
 /* administration struct */
 typedef struct zparser zparser_type;
@@ -78,6 +81,7 @@ void zc_error_prev_line(const char *fmt, ...) ATTR_FORMAT(printf, 1, 2);
 
 int process_rr(void);
 uint16_t *zparser_conv_hex(region_type *region, const char *hex, size_t len);
+uint16_t *zparser_conv_hex_length(region_type *region, const char *hex, size_t len);
 uint16_t *zparser_conv_time(region_type *region, const char *time);
 uint16_t *zparser_conv_services(region_type *region, const char *protostr, char *servicestr);
 uint16_t *zparser_conv_period(region_type *region, const char *periodstr);
@@ -87,6 +91,7 @@ uint16_t *zparser_conv_byte(region_type *region, const char *text);
 uint16_t *zparser_conv_a(region_type *region, const char *text);
 uint16_t *zparser_conv_aaaa(region_type *region, const char *text);
 uint16_t *zparser_conv_text(region_type *region, const char *text, size_t len);
+uint16_t *zparser_conv_b32(region_type *region, const char *b32);
 uint16_t *zparser_conv_b64(region_type *region, const char *b64);
 uint16_t *zparser_conv_rrtype(region_type *region, const char *rr);
 uint16_t *zparser_conv_nxt(region_type *region, uint8_t nxtbits[]);
@@ -102,10 +107,10 @@ void parse_unknown_rdata(uint16_t type, uint16_t *wireformat);
 int32_t zparser_ttl2int(const char *ttlstr);
 void zadd_rdata_wireformat(uint16_t *data);
 void zadd_rdata_domain(domain_type *domain);
-void zprintrr(FILE *f, rr_type *rr);
 
 void set_bitnsec(uint8_t  bits[NSEC_WINDOW_COUNT][NSEC_WINDOW_BITS_SIZE],
 		 uint16_t index);
+uint16_t *alloc_rdata_init(region_type *region, const void *data, size_t size);
 
 /* zparser.y */
 zparser_type *zparser_create(region_type *region, region_type *rr_region,
